@@ -204,8 +204,8 @@ static void xf_draw_screen_scaled(xfContext* xfc, int x, int y, int w, int h)
 	const double yScalingFactor = 1.0 *
 	                              freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight) /
 	                              (double)xfc->scaledHeight;
-	XSetFillStyle(xfc->display, xfc->gc, FillSolid);
-	XSetForeground(xfc->display, xfc->gc, 0);
+	LogDynAndXSetFillStyle(xfc->log, xfc->display, xfc->gc, FillSolid);
+	LogDynAndXSetForeground(xfc->log, xfc->display, xfc->gc, 0);
 	/* Black out possible space between desktop and window borders */
 	{
 		XRectangle box1 = { 0, 0, WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->width),
@@ -221,11 +221,11 @@ static void xf_draw_screen_scaled(xfContext* xfc, int x, int y, int w, int h)
 
 		if (XSubtractRegion(reg1, reg2, reg1) && !XEmptyRegion(reg1))
 		{
-			XSetRegion(xfc->display, xfc->gc, reg1);
-			XFillRectangle(xfc->display, xfc->window->handle, xfc->gc, 0, 0,
-			               WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->width),
-			               WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->height));
-			XSetClipMask(xfc->display, xfc->gc, None);
+			LogDynAndXSetRegion(xfc->log, xfc->display, xfc->gc, reg1);
+			LogDynAndXFillRectangle(xfc->log, xfc->display, xfc->window->handle, xfc->gc, 0, 0,
+			                        WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->width),
+			                        WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->height));
+			LogDynAndXSetClipMask(xfc->log, xfc->display, xfc->gc, None);
 		}
 
 		XDestroyRegion(reg1);
@@ -329,31 +329,31 @@ void xf_draw_screen_(xfContext* xfc, int x, int y, int w, int h, const char* fkt
 	}
 
 #endif
-	XCopyArea(xfc->display, xfc->primary, xfc->window->handle, xfc->gc, x, y,
-	          WINPR_ASSERTING_INT_CAST(uint32_t, w), WINPR_ASSERTING_INT_CAST(uint32_t, h), x, y);
+	LogDynAndXCopyArea(xfc->log, xfc->display, xfc->primary, xfc->window->handle, xfc->gc, x, y,
+	                   WINPR_ASSERTING_INT_CAST(uint32_t, w), WINPR_ASSERTING_INT_CAST(uint32_t, h),
+	                   x, y);
 }
 
 static BOOL xf_desktop_resize(rdpContext* context)
 {
-	rdpSettings* settings = NULL;
 	xfContext* xfc = (xfContext*)context;
 
 	WINPR_ASSERT(xfc);
 
-	settings = context->settings;
+	rdpSettings* settings = context->settings;
 	WINPR_ASSERT(settings);
 
 	if (xfc->primary)
 	{
 		BOOL same = (xfc->primary == xfc->drawing) ? TRUE : FALSE;
-		XFreePixmap(xfc->display, xfc->primary);
+		LogDynAndXFreePixmap(xfc->log, xfc->display, xfc->primary);
 
 		WINPR_ASSERT(xfc->depth != 0);
-		if (!(xfc->primary =
-		          XCreatePixmap(xfc->display, xfc->drawable,
-		                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
-		                        WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth))))
+		if (!(xfc->primary = LogDynAndXCreatePixmap(
+		          xfc->log, xfc->display, xfc->drawable,
+		          freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		          freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
+		          WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth))))
 			return FALSE;
 
 		if (same)
@@ -396,12 +396,12 @@ static BOOL xf_desktop_resize(rdpContext* context)
 			    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 		}
 
-		XSetFunction(xfc->display, xfc->gc, GXcopy);
-		XSetFillStyle(xfc->display, xfc->gc, FillSolid);
-		XSetForeground(xfc->display, xfc->gc, 0);
-		XFillRectangle(xfc->display, xfc->drawable, xfc->gc, 0, 0,
-		               freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		               freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+		LogDynAndXSetFunction(xfc->log, xfc->display, xfc->gc, GXcopy);
+		LogDynAndXSetFillStyle(xfc->log, xfc->display, xfc->gc, FillSolid);
+		LogDynAndXSetForeground(xfc->log, xfc->display, xfc->gc, 0);
+		LogDynAndXFillRectangle(xfc->log, xfc->display, xfc->drawable, xfc->gc, 0, 0,
+		                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 	}
 
 	return TRUE;
@@ -424,9 +424,10 @@ static BOOL xf_paint(xfContext* xfc, const GDI_RGN* region)
 	}
 	else
 	{
-		XPutImage(xfc->display, xfc->primary, xfc->gc, xfc->image, region->x, region->y, region->x,
-		          region->y, WINPR_ASSERTING_INT_CAST(UINT16, region->w),
-		          WINPR_ASSERTING_INT_CAST(UINT16, region->h));
+		LogDynAndXPutImage(xfc->log, xfc->display, xfc->primary, xfc->gc, xfc->image, region->x,
+		                   region->y, region->x, region->y,
+		                   WINPR_ASSERTING_INT_CAST(UINT16, region->w),
+		                   WINPR_ASSERTING_INT_CAST(UINT16, region->h));
 		xf_draw_screen(xfc, region->x, region->y, region->w, region->h);
 	}
 	return TRUE;
@@ -435,16 +436,22 @@ static BOOL xf_paint(xfContext* xfc, const GDI_RGN* region)
 static BOOL xf_end_paint(rdpContext* context)
 {
 	xfContext* xfc = (xfContext*)context;
+	WINPR_ASSERT(xfc);
+
 	rdpGdi* gdi = context->gdi;
+	WINPR_ASSERT(gdi);
 
 	if (gdi->suppressOutput)
 		return TRUE;
 
 	HGDI_DC hdc = gdi->primary->hdc;
+	if (!hdc->hwnd)
+		return TRUE;
 
+	HGDI_WND hwnd = hdc->hwnd;
 	if (!xfc->complex_regions)
 	{
-		const GDI_RGN* rgn = hdc->hwnd->invalid;
+		const GDI_RGN* rgn = hwnd->invalid;
 		if (rgn->null)
 			return TRUE;
 		xf_lock_x11(xfc);
@@ -454,10 +461,10 @@ static BOOL xf_end_paint(rdpContext* context)
 	}
 	else
 	{
-		const INT32 ninvalid = hdc->hwnd->ninvalid;
-		const GDI_RGN* cinvalid = hdc->hwnd->cinvalid;
+		const INT32 ninvalid = hwnd->ninvalid;
+		const GDI_RGN* cinvalid = hwnd->cinvalid;
 
-		if (hdc->hwnd->ninvalid < 1)
+		if (hwnd->ninvalid < 1)
 			return TRUE;
 
 		xf_lock_x11(xfc);
@@ -469,29 +476,39 @@ static BOOL xf_end_paint(rdpContext* context)
 				return FALSE;
 		}
 
-		XFlush(xfc->display);
+		LogDynAndXFlush(xfc->log, xfc->display);
 		xf_unlock_x11(xfc);
 	}
 
-	hdc->hwnd->invalid->null = TRUE;
-	hdc->hwnd->ninvalid = 0;
+	hwnd->invalid->null = TRUE;
+	hwnd->ninvalid = 0;
 	return TRUE;
 }
 
 static BOOL xf_sw_desktop_resize(rdpContext* context)
 {
+	WINPR_ASSERT(context);
+
 	rdpGdi* gdi = context->gdi;
+	WINPR_ASSERT(gdi);
+
 	xfContext* xfc = (xfContext*)context;
 	rdpSettings* settings = context->settings;
+	WINPR_ASSERT(settings);
+
 	BOOL ret = FALSE;
 
+	/* There is a possible race here.
+	 * Ensure that the drawing thread does not update the screen during a
+	 * resize. */
+	const BOOL suppress = gdi->suppressOutput;
+	gdi->suppressOutput = TRUE;
+
+	xf_lock_x11(xfc);
 	if (!gdi_resize(gdi, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
 	                freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)))
-		return FALSE;
+		goto out;
 
-	/* Do not lock during gdi_resize, there might still be drawing operations in progress.
-	 * locking will deadlock. */
-	xf_lock_x11(xfc);
 	if (xfc->image)
 	{
 		xfc->image->data = NULL;
@@ -499,9 +516,10 @@ static BOOL xf_sw_desktop_resize(rdpContext* context)
 	}
 
 	WINPR_ASSERT(xfc->depth != 0);
-	if (!(xfc->image = XCreateImage(
-	          xfc->display, xfc->visual, WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth), ZPixmap, 0,
-	          (char*)gdi->primary_buffer, WINPR_ASSERTING_INT_CAST(uint32_t, gdi->width),
+	if (!(xfc->image = LogDynAndXCreateImage(
+	          xfc->log, xfc->display, xfc->visual, WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth),
+	          ZPixmap, 0, (char*)gdi->primary_buffer,
+	          WINPR_ASSERTING_INT_CAST(uint32_t, gdi->width),
 	          WINPR_ASSERTING_INT_CAST(uint32_t, gdi->height), xfc->scanline_pad,
 	          WINPR_ASSERTING_INT_CAST(int, gdi->stride))))
 	{
@@ -513,6 +531,7 @@ static BOOL xf_sw_desktop_resize(rdpContext* context)
 	ret = xf_desktop_resize(context);
 out:
 	xf_unlock_x11(xfc);
+	gdi->suppressOutput = suppress;
 	return ret;
 }
 
@@ -691,30 +710,33 @@ BOOL xf_create_window(xfContext* xfc)
 	}
 
 	if (!xfc->gc)
-		xfc->gc = XCreateGC(xfc->display, xfc->drawable, GCGraphicsExposures, &gcv);
+		xfc->gc =
+		    LogDynAndXCreateGC(xfc->log, xfc->display, xfc->drawable, GCGraphicsExposures, &gcv);
 
 	WINPR_ASSERT(xfc->depth != 0);
 	if (!xfc->primary)
-		xfc->primary = XCreatePixmap(xfc->display, xfc->drawable,
-		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
-		                             WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth));
+		xfc->primary =
+		    LogDynAndXCreatePixmap(xfc->log, xfc->display, xfc->drawable,
+		                           freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		                           freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
+		                           WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth));
 
 	xfc->drawing = xfc->primary;
 
 	if (!xfc->bitmap_mono)
-		xfc->bitmap_mono = XCreatePixmap(xfc->display, xfc->drawable, 8, 8, 1);
+		xfc->bitmap_mono = LogDynAndXCreatePixmap(xfc->log, xfc->display, xfc->drawable, 8, 8, 1);
 
 	if (!xfc->gc_mono)
-		xfc->gc_mono = XCreateGC(xfc->display, xfc->bitmap_mono, GCGraphicsExposures, &gcv);
+		xfc->gc_mono =
+		    LogDynAndXCreateGC(xfc->log, xfc->display, xfc->bitmap_mono, GCGraphicsExposures, &gcv);
 
-	XSetFunction(xfc->display, xfc->gc, GXcopy);
-	XSetFillStyle(xfc->display, xfc->gc, FillSolid);
-	XSetForeground(xfc->display, xfc->gc, BlackPixelOfScreen(xfc->screen));
-	XFillRectangle(xfc->display, xfc->primary, xfc->gc, 0, 0,
-	               freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-	               freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
-	XFlush(xfc->display);
+	LogDynAndXSetFunction(xfc->log, xfc->display, xfc->gc, GXcopy);
+	LogDynAndXSetFillStyle(xfc->log, xfc->display, xfc->gc, FillSolid);
+	LogDynAndXSetForeground(xfc->log, xfc->display, xfc->gc, BlackPixelOfScreen(xfc->screen));
+	LogDynAndXFillRectangle(xfc->log, xfc->display, xfc->primary, xfc->gc, 0, 0,
+	                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+	                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+	LogDynAndXFlush(xfc->log, xfc->display);
 
 	return TRUE;
 }
@@ -729,12 +751,12 @@ BOOL xf_create_image(xfContext* xfc)
 		WINPR_ASSERT(cgdi);
 
 		WINPR_ASSERT(xfc->depth != 0);
-		xfc->image =
-		    XCreateImage(xfc->display, xfc->visual, WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth),
-		                 ZPixmap, 0, (char*)cgdi->primary_buffer,
-		                 freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		                 freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
-		                 xfc->scanline_pad, WINPR_ASSERTING_INT_CAST(int, cgdi->stride));
+		xfc->image = LogDynAndXCreateImage(
+		    xfc->log, xfc->display, xfc->visual, WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth),
+		    ZPixmap, 0, (char*)cgdi->primary_buffer,
+		    freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		    freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight), xfc->scanline_pad,
+		    WINPR_ASSERTING_INT_CAST(int, cgdi->stride));
 		xfc->image->byte_order = LSBFirst;
 		xfc->image->bitmap_bit_order = LSBFirst;
 	}
@@ -766,25 +788,25 @@ static void xf_window_free(xfContext* xfc)
 
 	if (xfc->bitmap_mono)
 	{
-		XFreePixmap(xfc->display, xfc->bitmap_mono);
+		LogDynAndXFreePixmap(xfc->log, xfc->display, xfc->bitmap_mono);
 		xfc->bitmap_mono = 0;
 	}
 
 	if (xfc->gc_mono)
 	{
-		XFreeGC(xfc->display, xfc->gc_mono);
+		LogDynAndXFreeGC(xfc->log, xfc->display, xfc->gc_mono);
 		xfc->gc_mono = 0;
 	}
 
 	if (xfc->primary)
 	{
-		XFreePixmap(xfc->display, xfc->primary);
+		LogDynAndXFreePixmap(xfc->log, xfc->display, xfc->primary);
 		xfc->primary = 0;
 	}
 
 	if (xfc->gc)
 	{
-		XFreeGC(xfc->display, xfc->gc);
+		LogDynAndXFreeGC(xfc->log, xfc->display, xfc->gc);
 		xfc->gc = 0;
 	}
 }
@@ -1553,11 +1575,7 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 	DWORD exit_code = 0;
 	DWORD waitStatus = 0;
 	HANDLE inputEvent = NULL;
-	HANDLE timer = NULL;
-	LARGE_INTEGER due = { 0 };
-	TimerEventArgs timerEvent = { 0 };
 
-	EventArgsInit(&timerEvent, "xfreerdp");
 	freerdp* instance = (freerdp*)param;
 	WINPR_ASSERT(instance);
 
@@ -1601,27 +1619,12 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 		goto disconnect;
 	}
 
-	timer = CreateWaitableTimerA(NULL, FALSE, "mainloop-periodic-timer");
-
-	if (!timer)
-	{
-		WLog_ERR(TAG, "failed to create timer");
-		goto disconnect;
-	}
-
-	due.QuadPart = 0;
-
-	if (!SetWaitableTimer(timer, &due, 20, NULL, NULL, FALSE))
-	{
-		goto disconnect;
-	}
 	inputEvent = xfc->x11event;
 
 	while (!freerdp_shall_disconnect_context(instance->context))
 	{
 		HANDLE handles[MAXIMUM_WAIT_OBJECTS] = { 0 };
 		DWORD nCount = 0;
-		handles[nCount++] = timer;
 		handles[nCount++] = inputEvent;
 
 		/*
@@ -1682,12 +1685,6 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 
 		if (!handle_window_events(instance))
 			break;
-
-		if ((waitStatus != WAIT_TIMEOUT) && (waitStatus == WAIT_OBJECT_0))
-		{
-			timerEvent.now = GetTickCount64();
-			PubSub_OnTimer(context->pubSub, context, &timerEvent);
-		}
 	}
 
 	if (!exit_code)
@@ -1706,9 +1703,6 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 
 disconnect:
 
-	if (timer)
-		(void)CloseHandle(timer);
-
 	freerdp_disconnect(instance);
 end:
 	ExitThread(exit_code);
@@ -1859,7 +1853,7 @@ void xf_teardown_x11(xfContext* xfc)
 
 	if (xfc->display)
 	{
-		XCloseDisplay(xfc->display);
+		LogDynAndXCloseDisplay(xfc->log, xfc->display);
 		xfc->display = NULL;
 	}
 
@@ -1947,9 +1941,9 @@ BOOL xf_setup_x11(xfContext* xfc)
 		unsigned long nitems = 0;
 		unsigned long after = 0;
 		unsigned char* data = NULL;
-		int status = LogTagAndXGetWindowProperty(
-		    TAG, xfc->display, RootWindowOfScreen(xfc->screen), xfc->NET_SUPPORTED, 0, 1024, False,
-		    XA_ATOM, &actual_type, &actual_format, &nitems, &after, &data);
+		int status = LogDynAndXGetWindowProperty(
+		    xfc->log, xfc->display, RootWindowOfScreen(xfc->screen), xfc->NET_SUPPORTED, 0, 1024,
+		    False, XA_ATOM, &actual_type, &actual_format, &nitems, &after, &data);
 
 		if ((status == Success) && (actual_type == XA_ATOM) && (actual_format == 32))
 		{
