@@ -221,6 +221,9 @@ static BOOL freerdp_assistance_crypt_derive_key_sha1(const BYTE* hash, size_t ha
 	BYTE pad1[64] = { 0 };
 	BYTE pad2[64] = { 0 };
 
+	if (hashLength == 0)
+		return FALSE;
+
 	memset(pad1, 0x36, sizeof(pad1));
 	memset(pad2, 0x5C, sizeof(pad2));
 
@@ -297,7 +300,8 @@ static BOOL freerdp_assistance_parse_address_list(rdpAssistanceFile* file, char*
 	char* s = ";";
 
 	// get the first token
-	char* token = strtok(strp, s);
+	char* saveptr = NULL;
+	char* token = strtok_s(strp, s, &saveptr);
 
 	// walk through other tokens
 	while (token != NULL)
@@ -311,7 +315,7 @@ static BOOL freerdp_assistance_parse_address_list(rdpAssistanceFile* file, char*
 		if (!append_address(file, token, port))
 			goto out;
 
-		token = strtok(NULL, s);
+		token = strtok_s(NULL, s, &saveptr);
 	}
 	rc = TRUE;
 out:
@@ -423,7 +427,9 @@ static BOOL freerdp_assistance_parse_attr(const char** opt, size_t* plength, con
 	char bkey[128] = { 0 };
 	const int rc = _snprintf(bkey, sizeof(bkey), "%s=\"", key);
 	WINPR_ASSERT(rc > 0);
-	WINPR_ASSERT(rc < sizeof(bkey));
+	WINPR_ASSERT((size_t)rc < sizeof(bkey));
+	if ((rc <= 0) || ((size_t)rc >= sizeof(bkey)))
+		return FALSE;
 
 	char* p = strstr(tag, bkey);
 	if (!p)
@@ -447,7 +453,7 @@ static BOOL freerdp_assistance_parse_attr(const char** opt, size_t* plength, con
 		         key);
 		return FALSE;
 	}
-	const size_t length = q - p;
+	const size_t length = WINPR_ASSERTING_INT_CAST(size_t, q - p);
 	*opt = p;
 	*plength = length;
 
@@ -492,13 +498,15 @@ static BOOL freerdp_assistance_parse_attr_uint32(UINT32* opt, const char* key, c
 		return FALSE;
 
 	char buffer[64] = { 0 };
-	if (size >= sizeof(buffer))
+	if ((!copt && (size > 0)) || (size >= sizeof(buffer)))
 	{
 		WLog_WARN(TAG, "Invalid UINT32 string '%s' [%" PRIuz "]", copt, size);
 		return FALSE;
 	}
 
-	strncpy(buffer, copt, size);
+	if (size > 0)
+		strncpy(buffer, copt, size);
+
 	errno = 0;
 	unsigned long val = strtoul(buffer, NULL, 0);
 
@@ -508,7 +516,7 @@ static BOOL freerdp_assistance_parse_attr_uint32(UINT32* opt, const char* key, c
 		return FALSE;
 	}
 
-	*opt = val;
+	*opt = (UINT32)val;
 
 	return TRUE;
 }
@@ -523,7 +531,9 @@ static char* freerdp_assistance_contains_element(char* input, size_t ilen, const
 	char bkey[128] = { 0 };
 	const int rc = _snprintf(bkey, sizeof(bkey), "<%s", key);
 	WINPR_ASSERT(rc > 0);
-	WINPR_ASSERT(rc < sizeof(bkey));
+	WINPR_ASSERT((size_t)rc < sizeof(bkey));
+	if ((rc < 0) || ((size_t)rc >= sizeof(bkey)))
+		return NULL;
 
 	char* tag = strstr(input, bkey);
 	if (!tag || (tag > input + ilen))
@@ -562,8 +572,10 @@ static char* freerdp_assistance_contains_element(char* input, size_t ilen, const
 		char ekey[128] = { 0 };
 		const int erc = _snprintf(ekey, sizeof(ekey), "</%s>", key);
 		WINPR_ASSERT(erc > 0);
-		WINPR_ASSERT(erc < sizeof(ekey));
-		const size_t offset = start - tag;
+		WINPR_ASSERT((size_t)erc < sizeof(ekey));
+		if ((erc <= 0) || ((size_t)erc >= sizeof(ekey)))
+			return NULL;
+		const size_t offset = WINPR_ASSERTING_INT_CAST(size_t, start - tag);
 		dend = end = strrstr(start, ilen - offset, ekey);
 		if (end)
 			end += strnlen(ekey, sizeof(ekey));
@@ -577,12 +589,12 @@ static char* freerdp_assistance_contains_element(char* input, size_t ilen, const
 		return NULL;
 	}
 	if (plen)
-		*plen = end - tag;
+		*plen = WINPR_ASSERTING_INT_CAST(size_t, end - tag);
 
 	if (pdata)
 		*pdata = data;
 	if (pdlen)
-		*pdlen = dend - data;
+		*pdlen = WINPR_ASSERTING_INT_CAST(size_t, dend - data);
 	return tag;
 }
 
@@ -634,7 +646,7 @@ static BOOL freerdp_assistance_get_element(char* input, size_t ilen, const char*
 
 	char* end = tag + len;
 	*element = data;
-	*elen = end - data + 1;
+	*elen = WINPR_ASSERTING_INT_CAST(size_t, end - data + 1);
 	return TRUE;
 }
 
@@ -663,7 +675,7 @@ static BOOL freerdp_assistance_parse_all_elements_of(rdpAssistanceFile* file, ch
 }
 
 static BOOL freerdp_assistance_parse_all_elements_of_l(rdpAssistanceFile* file, char* data,
-                                                       size_t len)
+                                                       WINPR_ATTR_UNUSED size_t len)
 {
 	UINT32 p = 0;
 	const char* n = NULL;
@@ -775,28 +787,23 @@ out_fail:
 
 char* freerdp_assistance_construct_expert_blob(const char* name, const char* pass)
 {
-	size_t size = 0;
-	size_t nameLength = 0;
-	size_t passLength = 0;
-	char* ExpertBlob = NULL;
-
 	if (!name || !pass)
 		return NULL;
 
-	nameLength = strlen(name) + strlen("NAME=");
-	passLength = strlen(pass) + strlen("PASS=");
-	size = nameLength + passLength + 64;
-	ExpertBlob = (char*)calloc(1, size);
+	const size_t nameLength = strlen(name) + strlen("NAME=");
+	const size_t passLength = strlen(pass) + strlen("PASS=");
+	const size_t size = nameLength + passLength + 64;
+	char* ExpertBlob = (char*)calloc(1, size);
 
 	if (!ExpertBlob)
 		return NULL;
 
-	(void)sprintf_s(ExpertBlob, size, "%" PRIdz ";NAME=%s%" PRIdz ";PASS=%s", nameLength, name,
+	(void)sprintf_s(ExpertBlob, size, "%" PRIuz ";NAME=%s%" PRIuz ";PASS=%s", nameLength, name,
 	                passLength, pass);
 	return ExpertBlob;
 }
 
-char* freerdp_assistance_generate_pass_stub(DWORD flags)
+char* freerdp_assistance_generate_pass_stub(WINPR_ATTR_UNUSED DWORD flags)
 {
 	UINT32 nums[14];
 	char* passStub = NULL;
@@ -846,36 +853,35 @@ BYTE* freerdp_assistance_encrypt_pass_stub(const char* password, const char* pas
 	BOOL rc = 0;
 	size_t cbPasswordW = 0;
 	size_t cbPassStubW = 0;
-	size_t EncryptedSize = 0;
-	BYTE PasswordHash[WINPR_MD5_DIGEST_LENGTH];
-	WINPR_CIPHER_CTX* rc4Ctx = NULL;
+	BYTE PasswordHash[WINPR_MD5_DIGEST_LENGTH] = { 0 };
+	WINPR_RC4_CTX* rc4Ctx = NULL;
 	BYTE* pbIn = NULL;
 	BYTE* pbOut = NULL;
-	size_t cbOut = 0;
-	size_t cbIn = 0;
-	size_t cbFinal = 0;
+	BYTE* res = NULL;
 	WCHAR* PasswordW = ConvertUtf8ToWCharAlloc(password, &cbPasswordW);
 	WCHAR* PassStubW = ConvertUtf8ToWCharAlloc(passStub, &cbPassStubW);
 
+	cbPasswordW = (cbPasswordW) * sizeof(WCHAR);
+	cbPassStubW = (cbPassStubW) * sizeof(WCHAR);
+	const size_t EncryptedSize = cbPassStubW + 4;
+
 	if (!PasswordW || !PassStubW)
 		goto fail;
 
-	cbPasswordW = (cbPasswordW) * sizeof(WCHAR);
-	cbPassStubW = (cbPassStubW) * sizeof(WCHAR);
 	if (!winpr_Digest(WINPR_MD_MD5, (BYTE*)PasswordW, cbPasswordW, (BYTE*)PasswordHash,
 	                  sizeof(PasswordHash)))
 		goto fail;
 
-	EncryptedSize = cbPassStubW + 4;
 	pbIn = (BYTE*)calloc(1, EncryptedSize);
 	pbOut = (BYTE*)calloc(1, EncryptedSize);
 
 	if (!pbIn || !pbOut)
 		goto fail;
 
-	*((UINT32*)pbIn) = (UINT32)cbPassStubW;
+	WINPR_ASSERT(cbPasswordW <= UINT32_MAX);
+	winpr_Data_Write_UINT32(pbIn, (UINT32)cbPassStubW);
 	CopyMemory(&pbIn[4], PassStubW, cbPassStubW);
-	rc4Ctx = winpr_Cipher_New(WINPR_CIPHER_ARC4_128, WINPR_ENCRYPT, PasswordHash, NULL);
+	rc4Ctx = winpr_RC4_New(PasswordHash, sizeof(PasswordHash));
 
 	if (!rc4Ctx)
 	{
@@ -883,35 +889,24 @@ BYTE* freerdp_assistance_encrypt_pass_stub(const char* password, const char* pas
 		goto fail;
 	}
 
-	cbOut = cbFinal = 0;
-	cbIn = EncryptedSize;
-	rc = winpr_Cipher_Update(rc4Ctx, pbIn, cbIn, pbOut, &cbOut);
+	rc = winpr_RC4_Update(rc4Ctx, EncryptedSize, pbIn, pbOut);
 
 	if (!rc)
 	{
 		WLog_ERR(TAG, "winpr_Cipher_Update failure");
 		goto fail;
 	}
-
-	if (!winpr_Cipher_Final(rc4Ctx, pbOut + cbOut, &cbFinal))
-	{
-		WLog_ERR(TAG, "winpr_Cipher_Final failure");
-		goto fail;
-	}
-
-	winpr_Cipher_Free(rc4Ctx);
-	free(pbIn);
-	free(PasswordW);
-	free(PassStubW);
-	*pEncryptedSize = EncryptedSize;
-	return pbOut;
+	res = pbOut;
 fail:
-	winpr_Cipher_Free(rc4Ctx);
+	winpr_RC4_Free(rc4Ctx);
 	free(PasswordW);
 	free(PassStubW);
 	free(pbIn);
-	free(pbOut);
-	return NULL;
+	if (!res)
+		free(pbOut);
+	else
+		*pEncryptedSize = EncryptedSize;
+	return res;
 }
 
 static BOOL freerdp_assistance_decrypt2(rdpAssistanceFile* file)
@@ -954,7 +949,8 @@ static BOOL freerdp_assistance_decrypt2(rdpAssistanceFile* file)
 		goto fail;
 
 	aesDec =
-	    winpr_Cipher_New(WINPR_CIPHER_AES_128_CBC, WINPR_DECRYPT, DerivedKey, InitializationVector);
+	    winpr_Cipher_NewEx(WINPR_CIPHER_AES_128_CBC, WINPR_DECRYPT, DerivedKey, sizeof(DerivedKey),
+	                       InitializationVector, sizeof(InitializationVector));
 
 	if (!aesDec)
 		goto fail;
@@ -1236,9 +1232,9 @@ int freerdp_assistance_parse_file(rdpAssistanceFile* file, const char* name, con
 		return -1;
 	}
 
-	_fseeki64(fp, 0, SEEK_END);
+	(void)_fseeki64(fp, 0, SEEK_END);
 	fileSize.i64 = _ftelli64(fp);
-	_fseeki64(fp, 0, SEEK_SET);
+	(void)_fseeki64(fp, 0, SEEK_SET);
 
 	if (fileSize.i64 < 1)
 	{
@@ -1342,8 +1338,14 @@ BOOL freerdp_assistance_populate_settings_from_assistance_file(rdpAssistanceFile
 	if (ports != addresses)
 		return FALSE;
 
-	const UINT32 port = (UINT32)ArrayList_GetItem(file->MachinePorts, 0);
-	if (!freerdp_settings_set_uint32(settings, FreeRDP_ServerPort, port))
+	union
+	{
+		uintptr_t port;
+		void* data;
+	} cnv;
+	cnv.data = ArrayList_GetItem(file->MachinePorts, 0);
+	WINPR_ASSERT(cnv.port <= UINT32_MAX);
+	if (!freerdp_settings_set_uint32(settings, FreeRDP_ServerPort, (UINT32)cnv.port))
 		return FALSE;
 
 	if (!freerdp_target_net_adresses_reset(settings, ports))
@@ -1351,8 +1353,10 @@ BOOL freerdp_assistance_populate_settings_from_assistance_file(rdpAssistanceFile
 
 	for (size_t x = 0; x < ports; x++)
 	{
-		const UINT32 mport = (UINT32)ArrayList_GetItem(file->MachinePorts, x);
-		if (!freerdp_settings_set_pointer_array(settings, FreeRDP_TargetNetPorts, x, &mport))
+		cnv.data = ArrayList_GetItem(file->MachinePorts, x);
+		WINPR_ASSERT(cnv.port <= UINT32_MAX);
+		const UINT32 port = (UINT32)cnv.port;
+		if (!freerdp_settings_set_pointer_array(settings, FreeRDP_TargetNetPorts, x, &port))
 			return FALSE;
 	}
 	for (size_t i = 0; i < addresses; i++)
@@ -1451,13 +1455,22 @@ void freerdp_assistance_print_file(rdpAssistanceFile* file, wLog* log, DWORD lev
 		const char* uri = NULL;
 		const char* addr = ArrayList_GetItem(file->MachineAddresses, x);
 		if (x < ArrayList_Count(file->MachinePorts))
-			port = (UINT32)ArrayList_GetItem(file->MachinePorts, x);
+		{
+			union
+			{
+				uintptr_t port;
+				void* data;
+			} cnv;
+			cnv.data = ArrayList_GetItem(file->MachinePorts, x);
+			WINPR_ASSERT(cnv.port <= UINT32_MAX);
+			port = (UINT32)cnv.port;
+		}
 		if (x < ArrayList_Count(file->MachineUris))
 			uri = ArrayList_GetItem(file->MachineUris, x);
 
-		WLog_Print(log, level, "MachineAddress [%" PRIdz ": %s", x, addr);
-		WLog_Print(log, level, "MachinePort    [%" PRIdz ": %" PRIu32, x, port);
-		WLog_Print(log, level, "MachineURI     [%" PRIdz ": %s", x, uri);
+		WLog_Print(log, level, "MachineAddress [%" PRIuz ": %s", x, addr);
+		WLog_Print(log, level, "MachinePort    [%" PRIuz ": %" PRIu32, x, port);
+		WLog_Print(log, level, "MachineURI     [%" PRIuz ": %s", x, uri);
 	}
 }
 
